package ru.zvzd3d.EKG.NikyEkgSD1;

import android.hardware.usb.UsbConstants;
import android.hardware.usb.UsbDevice;
import android.hardware.usb.UsbDeviceConnection;
import android.hardware.usb.UsbEndpoint;
import android.hardware.usb.UsbInterface;

public class OUsbComCH340 
{
	public UsbEndpoint m_EP_In = null;   // FindDataEPs
	public UsbEndpoint m_EP_Out = null;
	
	public static boolean IsMyDevice(UsbDevice device)
	{
		if (device == null) return false;
		if (device.getVendorId() == 0x4348 && device.getProductId() == 0x5523) return true;
		if (device.getVendorId() == 0x1a86 && device.getProductId() == 0x7523) return true;
		if (device.getVendorId() == 0x1a86 && device.getProductId() == 0x5523) return true;
		return false;
	};
	
	public boolean FindDataEPs(UsbInterface intf)
	{
	   m_EP_In = null;  m_EP_Out = null;
	   int nf = 0;
	   for (int i=0; i<intf.getEndpointCount(); i++)
       {
    	 UsbEndpoint ep = intf.getEndpoint(i);
    	 if (ep.getType() == UsbConstants.USB_ENDPOINT_XFER_BULK && ep.getDirection() == UsbConstants.USB_DIR_IN)
    	 {
    		m_EP_In = ep;  nf++; 
    	 };
    	 if (ep.getType() == UsbConstants.USB_ENDPOINT_XFER_BULK && ep.getDirection() == UsbConstants.USB_DIR_OUT)
    	 {
    		m_EP_Out = ep;  nf++; 
    	 };
       };
       if (nf != 2 || m_EP_In == null || m_EP_Out == null)
       {
    	 m_EP_In = null;  m_EP_Out = null;
    	 return false;
       };
       return true;
	};
	
	public String Init(UsbDeviceConnection UsbConn, UsbInterface intf, int Speed)
	{
	  //  ,   ,     open   
	  boolean IsIni = ch341_control_out(UsbConn, CH341_REQ_SERIAL_INIT, 0, 0);
	  if (!IsIni) return "CH340:  CH341_REQ_SERIAL_INIT";

	  int lcr = CH341_LCR_ENABLE_RX | CH341_LCR_ENABLE_TX; 
	  lcr |= CH341_LCR_CS8;   //  

	  boolean Is = ch341_set_baudrate_lcr(UsbConn, Speed, lcr);
	  if (!Is) return "CH340:  ch341_set_baudrate_lcr";

	  int mcr = 0;
	  Is = ch341_set_handshake(UsbConn, mcr);
	  if (!Is) return "CH340:  ch341_set_handshake";

  	  return "";	  
	};
	
	private boolean ch341_control_out(UsbDeviceConnection UsbConn, int SetupCode, int Value, int Index)
	{
	  int len = UsbConn.controlTransfer(UsbConstants.USB_TYPE_VENDOR | USB_RECIP_DEVICE | UsbConstants.USB_DIR_OUT, SetupCode, Value, Index, null, 0, 1000);
	  if (len < 0) return false;
	  return true;
	};
	
	private boolean ch341_set_baudrate_lcr(UsbDeviceConnection UsbConn, int Speed, int lcr)
	{
	  int val = ch341_get_divisor(Speed);
	  if (val < 0) return false;
	  
	  // CH341A buffers data until a full endpoint-size packet (32 bytes) has been received unless bit 7 is set.
	  val |= 128;
	  
	  boolean Is = ch341_control_out(UsbConn, CH341_REQ_WRITE_REG, 0x1312, val);
	  if (!Is) return false;

	  Is = ch341_control_out(UsbConn, CH341_REQ_WRITE_REG, 0x2518, lcr);  
	  if (!Is) return false;
	  
	  return true;
	};
	
	private boolean ch341_set_handshake(UsbDeviceConnection UsbConn, int control)
	{
	  return ch341_control_out(UsbConn, CH341_REQ_MODEM_CTRL, ~control, 0); 
	};
	
	private static int ch341_get_divisor(int Speed)
	{
		final int ps = 3;  //     (  )
		final int fact=1;
		int ST_baseClock = 12 - 3*ps - fact;
		double BaseF = 48000000.0 / (1<<ST_baseClock);
		
		int Div = (int) Math.floor(BaseF / Speed + 0.5);
		
		if (Div<1 || Div>255) return -1; //   ps  fact
		
		return ((0x100 - Div) << 8) | (fact << 2) | ps;	
	};

	
	
	
	// This bit flag is used in endpoint descriptors' bEndpointAddress field.
	// It's also one of three fields in control requests bRequestType.
	//
	//private static final int USB_DIR_OUT	=		   0;		// to device 
	//private static final int USB_DIR_IN		=	 0x80;		// to host
	// USB types, the second of three bRequestType fields
	//private static final int USB_TYPE_STANDARD	=	(0x00 << 5);    // 00
	//private static final int USB_TYPE_CLASS		=	(0x01 << 5);    // 20
	//private static final int USB_TYPE_VENDOR	=	    (0x02 << 5);    // 40

	// USB recipients, the third of three bRequestType fields
	private static final int USB_RECIP_DEVICE		=   0x00;
	private static final int USB_RECIP_INTERFACE	=	0x01;

	private static final int CH341_REQ_READ_VERSION = 0x5F;
	private static final int CH341_REQ_WRITE_REG    = 0x9A;
	private static final int CH341_REQ_READ_REG     = 0x95;
	private static final int CH341_REQ_SERIAL_INIT  = 0xA1;
	private static final int CH341_REQ_MODEM_CTRL   = 0xA4;

	private static final int CH341_REG_BREAK        = 0x05;
	private static final int CH341_REG_LCR          = 0x18;
	private static final int CH341_NBREAK_BITS      = 0x01;

	private static final int CH341_LCR_ENABLE_RX    = 0x80;
	private static final int CH341_LCR_ENABLE_TX    = 0x40;
	private static final int CH341_LCR_MARK_SPACE   = 0x20;
	private static final int CH341_LCR_PAR_EVEN     = 0x10;
	private static final int CH341_LCR_ENABLE_PAR   = 0x08;
	private static final int CH341_LCR_STOP_BITS_2  = 0x04;
	private static final int CH341_LCR_CS8          = 0x03;
	private static final int CH341_LCR_CS7          = 0x02;
	private static final int CH341_LCR_CS6          = 0x01;
	private static final int CH341_LCR_CS5          = 0x00;
}
